@@ -36,6 +36,18 @@
 
 #define SUPPORTED_MUTTER_REMOTE_DESKTOP_API_VERSION 1
 
+typedef enum _GnomeRemoteDesktopDeviceType
+{
+  GNOME_REMOTE_DESKTOP_DEVICE_TYPE_KEYBOARD = 1 << 0,
+  GNOME_REMOTE_DESKTOP_DEVICE_TYPE_POINTER = 1 << 1,
+  GNOME_REMOTE_DESKTOP_DEVICE_TYPE_TOUCHSCREEN = 1 << 2,
+} GnomeRemoteDesktopDeviceType;
+
+enum _GnomeRemoteDesktopNotifyAxisFlags
+{
+  GNOME_REMOTE_DESKTOP_NOTIFY_AXIS_FLAGS_FINISH = 1 << 0,
+} GnomeRemoteDesktopNotifyAxisFlags;
+
 typedef struct _RemoteDesktopDialogHandle RemoteDesktopDialogHandle;
 
 typedef struct _RemoteDesktopSession
@@ -669,13 +681,21 @@ handle_notify_pointer_axis (XdpImplRemoteDesktop *object,
 {
   RemoteDesktopSession *remote_desktop_session;
   OrgGnomeMutterRemoteDesktopSession *proxy;
+  gboolean finish;
+  unsigned int flags = 0;
 
   remote_desktop_session =
     (RemoteDesktopSession *)lookup_session (arg_session_handle);
   proxy = remote_desktop_session->mutter_session_proxy;
 
+  if (g_variant_lookup (arg_options, "finish", "b", &finish))
+    {
+      if (finish)
+        flags = GNOME_REMOTE_DESKTOP_NOTIFY_AXIS_FLAGS_FINISH;
+    }
+
   org_gnome_mutter_remote_desktop_session_call_notify_pointer_axis (proxy,
-                                                                    dx, dy,
+                                                                    dx, dy, flags,
                                                                     NULL, NULL, NULL);
 
   xdp_impl_remote_desktop_complete_notify_pointer_axis (object, invocation);
@@ -837,6 +857,21 @@ handle_notify_touch_up (XdpImplRemoteDesktop *object,
   return TRUE;
 }
 
+static unsigned int
+gnome_device_types_xdp_device_types (unsigned int gnome_device_types)
+{
+  unsigned int supported_device_types = REMOTE_DESKTOP_DEVICE_TYPE_NONE;
+
+  if (gnome_device_types & GNOME_REMOTE_DESKTOP_DEVICE_TYPE_POINTER)
+    supported_device_types |= REMOTE_DESKTOP_DEVICE_TYPE_POINTER;
+  if (gnome_device_types & GNOME_REMOTE_DESKTOP_DEVICE_TYPE_KEYBOARD)
+    supported_device_types |= REMOTE_DESKTOP_DEVICE_TYPE_KEYBOARD;
+  if (gnome_device_types & GNOME_REMOTE_DESKTOP_DEVICE_TYPE_TOUCHSCREEN)
+    supported_device_types |= REMOTE_DESKTOP_DEVICE_TYPE_TOUCHSCREEN;
+
+  return supported_device_types;
+}
+
 static void
 remote_desktop_name_appeared (GDBusConnection *connection,
                               const char *name,
@@ -845,6 +880,7 @@ remote_desktop_name_appeared (GDBusConnection *connection,
 {
   g_autoptr(GError) error = NULL;
   int api_version;
+  unsigned int supported_device_types;
 
   remote_desktop =
     org_gnome_mutter_remote_desktop_proxy_new_sync (impl_connection,
@@ -898,8 +934,11 @@ remote_desktop_name_appeared (GDBusConnection *connection,
   g_signal_connect (impl, "handle-notify-touch-up",
                     G_CALLBACK (handle_notify_touch_up), NULL);
 
+  supported_device_types =
+    org_gnome_mutter_remote_desktop_get_supported_device_types (remote_desktop);
   g_object_set (G_OBJECT (impl),
-                "available-device-types", REMOTE_DESKTOP_DEVICE_TYPE_ALL,
+                "available-device-types",
+                gnome_device_types_xdp_device_types (supported_device_types),
                 NULL);
 
   if (!g_dbus_interface_skeleton_export (impl,
